@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2002-2004 MySQL AB
+ Copyright (C) 2002-2007 MySQL AB
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License as 
@@ -26,6 +26,7 @@ package testsuite.regression;
 
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
+import java.sql.DriverPropertyInfo;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
@@ -38,6 +39,7 @@ import java.util.Properties;
 import testsuite.BaseTestCase;
 
 import com.mysql.jdbc.Driver;
+import com.mysql.jdbc.NonRegisteringDriver;
 import com.mysql.jdbc.SQLError;
 
 /**
@@ -735,10 +737,12 @@ public class MetaDataRegressionTest extends BaseTestCase {
 			assertEquals(16777215, rsmd.getColumnDisplaySize(4));
 
 			if (versionMeetsMinimum(4, 1)) {
-				// Server doesn't send us enough information to detect LONGBLOB type
+				// Server doesn't send us enough information to detect LONGBLOB
+				// type
 				assertEquals(Integer.MAX_VALUE, rsmd.getPrecision(5));
 				assertEquals(Types.LONGVARBINARY, rsmd.getColumnType(5));
-				assertTrue("LONGBLOB".equalsIgnoreCase(rsmd.getColumnTypeName(5)));
+				assertTrue("LONGBLOB".equalsIgnoreCase(rsmd
+						.getColumnTypeName(5)));
 				assertEquals(Integer.MAX_VALUE, rsmd.getColumnDisplaySize(5));
 			}
 		} finally {
@@ -792,12 +796,12 @@ public class MetaDataRegressionTest extends BaseTestCase {
 			this.rs = dbmd.getIndexInfo(this.conn.getCatalog(), null,
 					tableName, true, false);
 			assertTrue(!this.rs.next()); // there should be no rows that meet
-											// this requirement
+			// this requirement
 
 			this.rs = dbmd.getIndexInfo(this.conn.getCatalog(), null,
 					tableName, false, false);
 			assertTrue(this.rs.next()); // there should be one row that meets
-										// this requirement
+			// this requirement
 			assertTrue(!this.rs.next());
 
 		} finally {
@@ -814,51 +818,51 @@ public class MetaDataRegressionTest extends BaseTestCase {
 	 */
 	public void testBug7033() throws Exception {
 		if (false) { // disabled for now
-		Connection big5Conn = null;
-		Statement big5Stmt = null;
-		PreparedStatement big5PrepStmt = null;
+			Connection big5Conn = null;
+			Statement big5Stmt = null;
+			PreparedStatement big5PrepStmt = null;
 
-		String testString = "\u5957 \u9910";
+			String testString = "\u5957 \u9910";
 
-		try {
-			Properties props = new Properties();
-			props.setProperty("useUnicode", "true");
-			props.setProperty("characterEncoding", "Big5");
+			try {
+				Properties props = new Properties();
+				props.setProperty("useUnicode", "true");
+				props.setProperty("characterEncoding", "Big5");
 
-			big5Conn = getConnectionWithProps(props);
-			big5Stmt = big5Conn.createStatement();
+				big5Conn = getConnectionWithProps(props);
+				big5Stmt = big5Conn.createStatement();
 
-			byte[] foobar = testString.getBytes("Big5");
-			System.out.println(foobar);
+				byte[] foobar = testString.getBytes("Big5");
+				System.out.println(foobar);
 
-			this.rs = big5Stmt.executeQuery("select 1 as '\u5957 \u9910'");
-			String retrString = this.rs.getMetaData().getColumnName(1);
-			assertTrue(testString.equals(retrString));
+				this.rs = big5Stmt.executeQuery("select 1 as '\u5957 \u9910'");
+				String retrString = this.rs.getMetaData().getColumnName(1);
+				assertTrue(testString.equals(retrString));
 
-			big5PrepStmt = big5Conn
-					.prepareStatement("select 1 as '\u5957 \u9910'");
-			this.rs = big5PrepStmt.executeQuery();
-			retrString = this.rs.getMetaData().getColumnName(1);
-			assertTrue(testString.equals(retrString));
-		} finally {
-			if (this.rs != null) {
-				this.rs.close();
-				this.rs = null;
+				big5PrepStmt = big5Conn
+						.prepareStatement("select 1 as '\u5957 \u9910'");
+				this.rs = big5PrepStmt.executeQuery();
+				retrString = this.rs.getMetaData().getColumnName(1);
+				assertTrue(testString.equals(retrString));
+			} finally {
+				if (this.rs != null) {
+					this.rs.close();
+					this.rs = null;
+				}
+
+				if (big5Stmt != null) {
+					big5Stmt.close();
+
+				}
+
+				if (big5PrepStmt != null) {
+					big5PrepStmt.close();
+				}
+
+				if (big5Conn != null) {
+					big5Conn.close();
+				}
 			}
-
-			if (big5Stmt != null) {
-				big5Stmt.close();
-
-			}
-
-			if (big5PrepStmt != null) {
-				big5PrepStmt.close();
-			}
-
-			if (big5Conn != null) {
-				big5Conn.close();
-			}
-		}
 		}
 	}
 
@@ -880,13 +884,13 @@ public class MetaDataRegressionTest extends BaseTestCase {
 			this.rs = dbmd.getIndexInfo(this.conn.getCatalog(), null,
 					tableName, true, false);
 			assertTrue(this.rs.next()); // there should be one row that meets
-										// this requirement
+			// this requirement
 			assertEquals(this.rs.getBoolean("NON_UNIQUE"), false);
 
 			this.rs = dbmd.getIndexInfo(this.conn.getCatalog(), null,
 					tableName, false, false);
 			assertTrue(this.rs.next()); // there should be two rows that meets
-										// this requirement
+			// this requirement
 			assertEquals(this.rs.getBoolean("NON_UNIQUE"), false);
 			assertTrue(this.rs.next());
 			assertEquals(this.rs.getBoolean("NON_UNIQUE"), true);
@@ -1141,7 +1145,7 @@ public class MetaDataRegressionTest extends BaseTestCase {
 							.getSQLState());
 				} else {
 					throw sqlEx; // we shouldn't have gotten an exception
-									// here
+					// here
 				}
 			}
 
@@ -1157,11 +1161,12 @@ public class MetaDataRegressionTest extends BaseTestCase {
 	 * Tests fix for BUG#11575 -- DBMD.storesLower/Mixed/UpperIdentifiers()
 	 * reports incorrect values for servers deployed on Windows.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug11575() throws Exception {
-		DatabaseMetaData dbmd= this.conn.getMetaData();
-		
+		DatabaseMetaData dbmd = this.conn.getMetaData();
+
 		if (isServerRunningOnWindows()) {
 			assertEquals(true, dbmd.storesLowerCaseIdentifiers());
 			assertEquals(true, dbmd.storesLowerCaseQuotedIdentifiers());
@@ -1178,58 +1183,71 @@ public class MetaDataRegressionTest extends BaseTestCase {
 			assertEquals(true, dbmd.storesUpperCaseQuotedIdentifiers());
 		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#11781, foreign key information
-     * that is quoted is parsed incorrectly.
-     */
+	 * Tests fix for BUG#11781, foreign key information that is quoted is parsed
+	 * incorrectly.
+	 */
 	public void testBug11781() throws Exception {
+
+		if (versionMeetsMinimum(5, 1)) {
+			if (!versionMeetsMinimum(5, 2)) {
+				// server bug prevents this test from functioning
+				
+				return;
+			}
+		}
 		
-	      createTable("`app tab`", "( C1 int(11) NULL, INDEX NEWINX (C1), INDEX NEWINX2 (C1)) ENGINE = InnoDB CHECKSUM = 0 COMMENT = 'InnoDB free: 3072 kB; (`C1`) REFER`test/app tab`(`C1`)' PACK_KEYS = 0");
-	      
-	      this.stmt.executeUpdate("ALTER TABLE `app tab` ADD CONSTRAINT APPFK FOREIGN KEY (C1) REFERENCES `app tab` (C1)");
-	      
-	      /*
-	      this.rs = this.conn.getMetaData().getCrossReference(
-	    		  this.conn.getCatalog(), 
-	    		  null, 
-	    		  "app tab", 
-	    		  this.conn.getCatalog(), 
-	    		  null, 
-	    		  "app tab");
-	    		  */
-	      this.rs = ((com.mysql.jdbc.DatabaseMetaData)this.conn.getMetaData()).extractForeignKeyFromCreateTable(this.conn.getCatalog(),"app tab");
-	     assertTrue("must return a row",this.rs.next()) ;
-	     
-	     String catalog = this.conn.getCatalog();
-	     
-	     assertEquals("comment; APPFK(`C1`) REFER `" + catalog + "`/ `app tab` (`C1`)", this.rs.getString(3));
-	     
-	     this.rs.close();
-	     
-	     this.rs = this.conn.getMetaData().getImportedKeys(this.conn.getCatalog(), null, "app tab");
-	     
-	     assertTrue(this.rs.next());
-	     
-	     this.rs = this.conn.getMetaData().getExportedKeys(this.conn.getCatalog(), null, "app tab");
-	     
-	     assertTrue(this.rs.next());
+		createTable(
+				"`app tab`",
+				"( C1 int(11) NULL, INDEX NEWINX (C1), INDEX NEWINX2 (C1)) ENGINE = InnoDB CHECKSUM = 0 COMMENT = 'InnoDB free: 3072 kB; (`C1`) REFER`test/app tab`(`C1`)' PACK_KEYS = 0");
+
+		this.stmt
+				.executeUpdate("ALTER TABLE `app tab` ADD CONSTRAINT APPFK FOREIGN KEY (C1) REFERENCES `app tab` (C1)");
+
+		/*
+		 * this.rs = this.conn.getMetaData().getCrossReference(
+		 * this.conn.getCatalog(), null, "app tab", this.conn.getCatalog(),
+		 * null, "app tab");
+		 */
+		this.rs = ((com.mysql.jdbc.DatabaseMetaData) this.conn.getMetaData())
+				.extractForeignKeyFromCreateTable(this.conn.getCatalog(),
+						"app tab");
+		assertTrue("must return a row", this.rs.next());
+
+		String catalog = this.conn.getCatalog();
+
+		assertEquals("comment; APPFK(`C1`) REFER `" + catalog
+				+ "`/ `app tab` (`C1`)", this.rs.getString(3));
+
+		this.rs.close();
+
+		this.rs = this.conn.getMetaData().getImportedKeys(
+				this.conn.getCatalog(), null, "app tab");
+
+		assertTrue(this.rs.next());
+
+		this.rs = this.conn.getMetaData().getExportedKeys(
+				this.conn.getCatalog(), null, "app tab");
+
+		assertTrue(this.rs.next());
 	}
-	
+
 	/**
-	 * Tests fix for BUG#12970 - java.sql.Types.OTHER returned for 
-	 * binary and varbinary columns.
-	 *
+	 * Tests fix for BUG#12970 - java.sql.Types.OTHER returned for binary and
+	 * varbinary columns.
+	 * 
 	 */
 	public void testBug12970() throws Exception {
 		if (versionMeetsMinimum(5, 0, 8)) {
 			String tableName = "testBug12970";
-		
-			createTable(tableName, "(binary_field BINARY(32), varbinary_field VARBINARY(64))");
-			
+
+			createTable(tableName,
+					"(binary_field BINARY(32), varbinary_field VARBINARY(64))");
+
 			try {
-				this.rs = this.conn.getMetaData().getColumns(this.conn.getCatalog(),
-						null, tableName, "%");
+				this.rs = this.conn.getMetaData().getColumns(
+						this.conn.getCatalog(), null, tableName, "%");
 				assertTrue(this.rs.next());
 				assertEquals(Types.BINARY, this.rs.getInt("DATA_TYPE"));
 				assertEquals(32, this.rs.getInt("COLUMN_SIZE"));
@@ -1237,9 +1255,10 @@ public class MetaDataRegressionTest extends BaseTestCase {
 				assertEquals(Types.VARBINARY, this.rs.getInt("DATA_TYPE"));
 				assertEquals(64, this.rs.getInt("COLUMN_SIZE"));
 				this.rs.close();
-				
-				this.rs = this.stmt.executeQuery("SELECT binary_field, varbinary_field FROM " 
-						+ tableName);
+
+				this.rs = this.stmt
+						.executeQuery("SELECT binary_field, varbinary_field FROM "
+								+ tableName);
 				ResultSetMetaData rsmd = this.rs.getMetaData();
 				assertEquals(Types.BINARY, rsmd.getColumnType(1));
 				assertEquals(32, rsmd.getPrecision(1));
@@ -1253,38 +1272,41 @@ public class MetaDataRegressionTest extends BaseTestCase {
 			}
 		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#12975 - OpenOffice expects DBMD.supportsIEF()
-	 * to return "true" if foreign keys are supported by the datasource,
-	 * even though this method also covers support for check constraints,
-	 * which MySQL _doesn't_ have.
+	 * Tests fix for BUG#12975 - OpenOffice expects DBMD.supportsIEF() to return
+	 * "true" if foreign keys are supported by the datasource, even though this
+	 * method also covers support for check constraints, which MySQL _doesn't_
+	 * have.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug12975() throws Exception {
-		assertEquals(false, this.conn.getMetaData().supportsIntegrityEnhancementFacility());
-		
+		assertEquals(false, this.conn.getMetaData()
+				.supportsIntegrityEnhancementFacility());
+
 		Connection overrideConn = null;
-		
+
 		try {
 			Properties props = new Properties();
-			
+
 			props.setProperty("overrideSupportsIntegrityEnhancementFacility",
 					"true");
-			
+
 			overrideConn = getConnectionWithProps(props);
-			assertEquals(true, overrideConn.getMetaData().supportsIntegrityEnhancementFacility());
+			assertEquals(true, overrideConn.getMetaData()
+					.supportsIntegrityEnhancementFacility());
 		} finally {
 			if (overrideConn != null) {
 				overrideConn.close();
 			}
 		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#13277 - RSMD for generated keys
-	 * has NPEs when a connection is referenced.
+	 * Tests fix for BUG#13277 - RSMD for generated keys has NPEs when a
+	 * connection is referenced.
 	 * 
 	 * @throws Exception
 	 */
@@ -1292,120 +1314,129 @@ public class MetaDataRegressionTest extends BaseTestCase {
 		if (isRunningOnJdk131()) {
 			return; // test not valid on JDK-1.3.1
 		}
-		
-		createTable("testBug13277", "(field1 INT NOT NULL PRIMARY KEY AUTO_INCREMENT, field2 VARCHAR(32))");
-		
+
+		createTable("testBug13277",
+				"(field1 INT NOT NULL PRIMARY KEY AUTO_INCREMENT, field2 VARCHAR(32))");
+
 		try {
-			this.stmt.executeUpdate("INSERT INTO testBug13277 (field2) VALUES ('abcdefg')",
+			this.stmt.executeUpdate(
+					"INSERT INTO testBug13277 (field2) VALUES ('abcdefg')",
 					Statement.RETURN_GENERATED_KEYS);
-			
+
 			this.rs = this.stmt.getGeneratedKeys();
-			
+
 			ResultSetMetaData rsmd = this.rs.getMetaData();
 			checkRsmdForBug13277(rsmd);
 			this.rs.close();
-			
+
 			for (int i = 0; i < 5; i++) {
-				this.stmt.addBatch("INSERT INTO testBug13277 (field2) VALUES ('abcdefg')");
+				this.stmt
+						.addBatch("INSERT INTO testBug13277 (field2) VALUES ('abcdefg')");
 			}
-			
+
 			this.stmt.executeBatch();
-			
+
 			this.rs = this.stmt.getGeneratedKeys();
-			
+
 			rsmd = this.rs.getMetaData();
 			checkRsmdForBug13277(rsmd);
 			this.rs.close();
-			
+
 			this.pstmt = this.conn.prepareStatement(
 					"INSERT INTO testBug13277 (field2) VALUES ('abcdefg')",
 					Statement.RETURN_GENERATED_KEYS);
 			this.pstmt.executeUpdate();
-			
+
 			this.rs = this.pstmt.getGeneratedKeys();
-			
+
 			rsmd = this.rs.getMetaData();
 			checkRsmdForBug13277(rsmd);
 			this.rs.close();
-			
+
 			this.pstmt.addBatch();
 			this.pstmt.addBatch();
-			
+
 			this.pstmt.executeUpdate();
-			
+
 			this.rs = this.pstmt.getGeneratedKeys();
-			
+
 			rsmd = this.rs.getMetaData();
 			checkRsmdForBug13277(rsmd);
 			this.rs.close();
-			
+
 		} finally {
 			if (this.pstmt != null) {
 				this.pstmt.close();
 				this.pstmt = null;
 			}
-			
+
 			if (this.rs != null) {
 				this.rs.close();
 				this.rs = null;
 			}
 		}
 	}
-	
+
 	/**
-	 * Tests BUG13601 (which doesn't seem to be present in 3.1.11,
-	 * but we'll leave it in here for regression's-sake).
+	 * Tests BUG13601 (which doesn't seem to be present in 3.1.11, but we'll
+	 * leave it in here for regression's-sake).
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug13601() throws Exception {
-		
+
 		if (versionMeetsMinimum(5, 0)) {
-			createTable("testBug13601", "(field1 BIGINT NOT NULL, field2 BIT default 0 NOT NULL) ENGINE=MyISAM");
-			
-			this.rs = this.stmt.executeQuery("SELECT field1, field2 FROM testBug13601 WHERE 1=-1");
+			createTable("testBug13601",
+					"(field1 BIGINT NOT NULL, field2 BIT default 0 NOT NULL) ENGINE=MyISAM");
+
+			this.rs = this.stmt
+					.executeQuery("SELECT field1, field2 FROM testBug13601 WHERE 1=-1");
 			ResultSetMetaData rsmd = this.rs.getMetaData();
 			assertEquals(Types.BIT, rsmd.getColumnType(2));
 			assertEquals(Boolean.class.getName(), rsmd.getColumnClassName(2));
-			
-			this.rs = this.conn.prepareStatement("SELECT field1, field2 FROM testBug13601 WHERE 1=-1").executeQuery();
+
+			this.rs = this.conn.prepareStatement(
+					"SELECT field1, field2 FROM testBug13601 WHERE 1=-1")
+					.executeQuery();
 			rsmd = this.rs.getMetaData();
 			assertEquals(Types.BIT, rsmd.getColumnType(2));
 			assertEquals(Boolean.class.getName(), rsmd.getColumnClassName(2));
-			
+
 		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#14815 - DBMD.getColumns() doesn't
-	 * return TABLE_NAME correctly.
+	 * Tests fix for BUG#14815 - DBMD.getColumns() doesn't return TABLE_NAME
+	 * correctly.
 	 * 
-	 * @throws Exception if the test fails
+	 * @throws Exception
+	 *             if the test fails
 	 */
 	public void testBug14815() throws Exception {
 		try {
 			createTable("testBug14815_1", "(field_1_1 int)");
 			createTable("testBug14815_2", "(field_2_1 int)");
-			
-			boolean lcTableNames = 
-				this.conn.getMetaData().storesLowerCaseIdentifiers();
-			
-			String tableName1 = lcTableNames ? "testbug14815_1" : "testBug14815_1";
-			String tableName2 = lcTableNames ? "testbug14815_2" : "testBug14815_2";
-			
+
+			boolean lcTableNames = this.conn.getMetaData()
+					.storesLowerCaseIdentifiers();
+
+			String tableName1 = lcTableNames ? "testbug14815_1"
+					: "testBug14815_1";
+			String tableName2 = lcTableNames ? "testbug14815_2"
+					: "testBug14815_2";
+
 			this.rs = this.conn.getMetaData().getColumns(
-					this.conn.getCatalog(), null, 
-					"testBug14815%", "%");
-			
+					this.conn.getCatalog(), null, "testBug14815%", "%");
+
 			assertTrue(this.rs.next());
 			assertEquals(tableName1, this.rs.getString("TABLE_NAME"));
 			assertEquals("field_1_1", this.rs.getString("COLUMN_NAME"));
-			
+
 			assertTrue(this.rs.next());
 			assertEquals(tableName2, this.rs.getString("TABLE_NAME"));
 			assertEquals("field_2_1", this.rs.getString("COLUMN_NAME"));
-			
-			
+
 		} finally {
 			if (this.rs != null) {
 				this.rs.close();
@@ -1413,12 +1444,12 @@ public class MetaDataRegressionTest extends BaseTestCase {
 			}
 		}
 	}
-	
-	/** 
-	 * Tests fix for BUG#15854 - DBMD.getColumns() returns wrong type
-	 * for BIT.
+
+	/**
+	 * Tests fix for BUG#15854 - DBMD.getColumns() returns wrong type for BIT.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug15854() throws Exception {
 		if (versionMeetsMinimum(5, 0)) {
@@ -1435,41 +1466,31 @@ public class MetaDataRegressionTest extends BaseTestCase {
 					toClose.close();
 				}
 			}
-			
+
 		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#16277 - Invalid classname returned for 
+	 * Tests fix for BUG#16277 - Invalid classname returned for
 	 * RSMD.getColumnClassName() for BIGINT type.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug16277() throws Exception {
 		createTable("testBug16277", "(field1 BIGINT, field2 BIGINT UNSIGNED)");
-		ResultSetMetaData rsmd = this.stmt.executeQuery("SELECT field1, field2 FROM testBug16277").getMetaData();
+		ResultSetMetaData rsmd = this.stmt.executeQuery(
+				"SELECT field1, field2 FROM testBug16277").getMetaData();
 		assertEquals("java.lang.Long", rsmd.getColumnClassName(1));
 		assertEquals("java.math.BigInteger", rsmd.getColumnClassName(2));
 	}
-	
+
 	/**
-	 * Tests fix for BUG#18258 - Nonexistent catalog/database causes SQLException
-	 * to be raised, rather than returning empty result set.
+	 * Tests fix for BUG#18554 - Aliased column names where length of name > 251
+	 * are corrupted.
 	 * 
-	 * @throws Exception if the test fails.
-	 */
-	public void testBug18258() throws Exception {
-		String bogusDatabaseName = "abcdefghijklmnopqrstuvwxyz";
-		this.conn.getMetaData().getTables(bogusDatabaseName, "%", "%", new String[] {"TABLE", "VIEW"});
-		this.conn.getMetaData().getColumns(bogusDatabaseName, "%", "%", "%");
-		this.conn.getMetaData().getProcedures(bogusDatabaseName, "%", "%");
-	}
-	
-	/**
-	 * Tests fix for BUG#18554 - Aliased column names where
-	 * length of name > 251 are corrupted.
-	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug18554() throws Exception {
 		testBug18554(249);
@@ -1480,101 +1501,209 @@ public class MetaDataRegressionTest extends BaseTestCase {
 		testBug18554(254);
 		testBug18554(255);
 	}
-	
-	/*
-	public void testBug20491() throws Exception {
-		createTable("testBug20491",  "("
-				 + "`field1_ae_` int(10) unsigned NOT NULL default '0',"
-				 + "`field2_ue_` varchar(45) default '',"
-				 + "`field3_oe_` varchar(45) default '',"
-				 + "`field4_sz_` varchar(45) default '',"
-				 + "PRIMARY KEY  (`field1_ae_`))"
-				);
-		this.rs = this.conn.getMetaData().getColumns(this.conn.getCatalog(), 
-				null, "testBug20491", "%");
-		this.rs.next();
-		assertEquals("field1_ae_", this.rs.getString("COLUMN_NAME"));
-	}*/
-	
-	public void testBug20191() throws Exception {
-		if (versionMeetsMinimum(5, 0)) {
-			stmt.executeUpdate("drop table if exists t1;");
-		    stmt.executeUpdate("create table t1 (id int(10))");
-		    stmt.executeUpdate("insert into t1 values (1);");
-		    stmt.executeUpdate("drop view if exists v1");
-		    stmt.executeUpdate("CREATE ALGORITHM=UNDEFINED SQL SECURITY DEFINER VIEW v1 AS select t1.id as id from t1");
-		    ResultSet rs = stmt.executeQuery("select * from v1 group by id limit 1");
-		    System.out.println("when limit is 1, getTableName gives: ");
-		    ResultSetMetaData rsmd = rs.getMetaData();
-		    System.out.println(rsmd.getTableName(1));
-		    System.out.println("<P>");
-		    rs = stmt.executeQuery("select * from v1 group by id limit 0");
-		    System.out.println("when limit is 0, getTableName gives: ");
-		    rsmd = rs.getMetaData();
-		    System.out.println(rsmd.getTableName(1));
-		    System.out.println("<P>");
-		    rs = stmt.executeQuery("select * from v1 where id=1000 group by id ");
-		    System.out.println("when result not found, getTableName gives: ");
-		    rsmd = rs.getMetaData();
-		    System.out.println(rsmd.getTableName(1));
-		    System.out.println("<P>");
-		    rs = stmt.executeQuery("select * from v1 where id=1 group by id ");
-		    System.out.println("when result not found, getTableName gives: ");
-		    rsmd = rs.getMetaData();
-		    System.out.println(rsmd.getTableName(1));
-		    System.out.println("<P>");
-		    System.out.println("Note that \"group by\" is essential in this case");
-		    rs.close();
-		}
-	}
+
 	private void testBug18554(int columnNameLength) throws Exception {
 		StringBuffer buf = new StringBuffer(columnNameLength + 2);
-		
-		for (int i = 0; i < columnNameLength; i++) {
-			buf.append((char)((Math.random() * 26) + 65));
-		}
-		
-		
-		String colName = buf.toString();
-		this.rs = this.stmt.executeQuery("select curtime() as `" + colName + "`");
-						ResultSetMetaData meta = this.rs.getMetaData();
 
-		assertEquals(colName, meta.getColumnName(1));
+		for (int i = 0; i < columnNameLength; i++) {
+			buf.append((char) ((Math.random() * 26) + 65));
+		}
+
+		String colName = buf.toString();
+		this.rs = this.stmt.executeQuery("select curtime() as `" + colName
+				+ "`");
+		ResultSetMetaData meta = this.rs.getMetaData();
+
+		assertEquals(colName, meta.getColumnLabel(1));
 
 	}
-	
-	private void checkRsmdForBug13277(ResultSetMetaData rsmd) throws SQLException {
+
+	private void checkRsmdForBug13277(ResultSetMetaData rsmd)
+			throws SQLException {
 		assertEquals(17, rsmd.getColumnDisplaySize(1));
-		
+
 		if (versionMeetsMinimum(4, 1)) {
 			assertEquals(false, rsmd.isDefinitelyWritable(1));
 			assertEquals(true, rsmd.isReadOnly(1));
 			assertEquals(false, rsmd.isWritable(1));
 		}
 	}
-	
+
 	public void testSupportsCorrelatedSubqueries() throws Exception {
 		DatabaseMetaData dbmd = this.conn.getMetaData();
-		
-		assertEquals(versionMeetsMinimum(4, 1), dbmd.supportsCorrelatedSubqueries());
+
+		assertEquals(versionMeetsMinimum(4, 1), dbmd
+				.supportsCorrelatedSubqueries());
 	}
-	
+
 	public void testSupportesGroupByUnrelated() throws Exception {
 		DatabaseMetaData dbmd = this.conn.getMetaData();
-		
+
 		assertEquals(true, dbmd.supportsGroupByUnrelated());
 	}
 	
-	public void testRSMDToStringFromDBMD() throws Exception {
-		try {		
-			this.rs = this.conn.getMetaData().getTypeInfo();
+	/**
+	 * Tests fix for BUG#21267, ParameterMetaData throws NullPointerException
+	 * when prepared SQL actually has a syntax error
+	 * 
+	 * @throws Exception
+	 */
+	public void testBug21267() throws Exception {
+		if (isRunningOnJdk131()) {
+			return; // no parameter metadata on JDK-1.3.1
+		}
+		
+		createTable(
+				"bug21267",
+				"(`Col1` int(11) NOT NULL,`Col2` varchar(45) default NULL,`Col3` varchar(45) default NULL,PRIMARY KEY  (`Col1`))");
+
+		try {
+			this.pstmt = this.conn
+					.prepareStatement("SELECT Col1, Col2,Col4 FROM bug21267 WHERE Col1=?");
+			this.pstmt.setInt(1, 1);
+
+			java.sql.ParameterMetaData psMeta = this.pstmt
+					.getParameterMetaData();
+
+			try {
+				assertEquals(0, psMeta.getParameterType(1));
+			} catch (SQLException sqlEx) {
+				assertEquals(SQLError.SQL_STATE_DRIVER_NOT_CAPABLE, sqlEx.getSQLState());
+			}
 			
-			this.rs.getMetaData().toString(); // used to cause NPE
+			this.pstmt.close();
+			
+			Properties props = new Properties();
+			props.setProperty("generateSimpleParameterMetadata", "true");
+			
+			this.pstmt = getConnectionWithProps(props).prepareStatement("SELECT Col1, Col2,Col4 FROM bug21267 WHERE Col1=?");
+			
+			psMeta = this.pstmt.getParameterMetaData();
+			
+			assertEquals(Types.VARCHAR, psMeta.getParameterType(1));
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
+
+	/**
+	 * Tests fix for BUG#21544 - When using information_schema for metadata, 
+	 * COLUMN_SIZE for getColumns() is not clamped to range of 
+	 * java.lang.Integer as is the case when not using 
+	 * information_schema, thus leading to a truncation exception that 
+	 * isn't present when not using information_schema.
+	 * 
+	 * @throws Exception if the test fails
+	 */
+	public void testBug21544() throws Exception {
+		if (!versionMeetsMinimum(5, 0)) {
+			return;
+		}
+		
+		createTable("testBug21544",
+                "(foo_id INT NOT NULL, stuff LONGTEXT"
+                + ", PRIMARY KEY (foo_id)) TYPE=INNODB");
+		
+		Connection infoSchemConn = null;
+		
+		Properties props = new Properties();
+		props.setProperty("useInformationSchema", "true");
+		props.setProperty("jdbcCompliantTruncation", "false");
+		
+		infoSchemConn = getConnectionWithProps(props);
+		
+		try {
+	        this.rs = infoSchemConn.getMetaData().getColumns(null, null, 
+	        		"testBug21544",
+	                null);
+	        
+	        while (rs.next()) {
+	        	rs.getInt("COLUMN_SIZE");   
+	        }
+	    } finally {
+            if (infoSchemConn != null) {
+            	infoSchemConn.close();
+            }
+            
+            closeMemberJDBCResources();
+        }
+	}
+
+	/** 
+	 * Tests fix for BUG#22613 - DBMD.getColumns() does not return expected
+	 * COLUMN_SIZE for the SET type (fixed to be consistent with the ODBC driver)
+	 * 
+	 * @throws Exception if the test fails
+	 */
+	public void testBug22613() throws Exception {
+		
+		createTable("bug22613", "( s set('a','bc','def','ghij') default NULL, t enum('a', 'ab', 'cdef'))");
+
+		try {
+			checkMetadataForBug22613(this.conn);
+			
+			if (versionMeetsMinimum(5, 0)) {
+				Connection infoSchemConn = null;
+			
+				try {
+					Properties props = new Properties();
+					props.setProperty("useInformationSchema", "true");
+					
+					infoSchemConn = getConnectionWithProps(props);
+					
+					checkMetadataForBug22613(infoSchemConn);
+				} finally {
+					if (infoSchemConn != null) {
+						infoSchemConn.close();
+					}
+				}
+			}
 		} finally {
 			closeMemberJDBCResources();
 		}
 	}
 	
+	private void checkMetadataForBug22613(Connection c) throws Exception {
+		String maxValue = "a,bc,def,ghij";
+		
+		try {
+			DatabaseMetaData meta = c.getMetaData();
+			this.rs = meta.getColumns(null, this.conn.getCatalog(), "bug22613", "s");
+			this.rs.first();
+
+			assertEquals(maxValue.length(), rs.getInt("COLUMN_SIZE"));
+			
+			this.rs = meta.getColumns(null, c.getCatalog(), "bug22613", "t");
+			this.rs.first();
+
+			assertEquals(4, rs.getInt("COLUMN_SIZE"));			
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
+
+	/**
+	 * Fix for BUG#22628 - Driver.getPropertyInfo() throws NullPointerException for URL that only specifies
+	 * host and/or port.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug22628() throws Exception {
+		DriverPropertyInfo[] dpi = new NonRegisteringDriver().getPropertyInfo("jdbc:mysql://bogus:9999", 
+				new Properties());
+		
+		boolean foundHost = false;
+		
+		for (int i = 0; i < dpi.length; i++) {
+			if ("bogus:9999".equals(dpi[i].value)) {
+				foundHost = true;
+				break;
+			}
+		}
+		
+		assertTrue(foundHost);
+	}
+
 	private void testAbsenceOfMetadataForQuery(String query) throws Exception {
 		try {
 			this.pstmt = this.conn.prepareStatement(query);
@@ -1593,6 +1722,233 @@ public class MetaDataRegressionTest extends BaseTestCase {
 			}
 		}
 	}
+
+	public void testRSMDToStringFromDBMD() throws Exception {
+		try {		
+			this.rs = this.conn.getMetaData().getTypeInfo();
+			
+			this.rs.getMetaData().toString(); // used to cause NPE
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
 	
+	public void testCharacterSetForDBMD() throws Exception {
+		if (versionMeetsMinimum(4, 0)) {
+			// server is broken, fixed in 5.2/6.0?
+			
+			if (!versionMeetsMinimum(5, 2)) {
+				return;
+			}
+		}
+		
+		String quoteChar = this.conn.getMetaData().getIdentifierQuoteString();
+		
+		String tableName = quoteChar + "\u00e9\u0074\u00e9" + quoteChar;
+		createTable(tableName, "(field1 int)");
+		this.rs = this.conn.getMetaData().getTables(this.conn.getCatalog(), 
+				null, tableName, new String[] {"TABLE"});
+		assertEquals(true, this.rs.next());
+		System.out.println(this.rs.getString("TABLE_NAME"));
+		System.out.println(new String(this.rs.getBytes("TABLE_NAME"), "UTF-8"));
+	}
+
+	/**
+	 * Tests fix for BUG#18258 - Nonexistent catalog/database causes SQLException
+	 * to be raised, rather than returning empty result set.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug18258() throws Exception {
+		String bogusDatabaseName = "abcdefghijklmnopqrstuvwxyz";
+		this.conn.getMetaData().getTables(bogusDatabaseName, "%", "%", new String[] {"TABLE", "VIEW"});
+		this.conn.getMetaData().getColumns(bogusDatabaseName, "%", "%", "%");
+		this.conn.getMetaData().getProcedures(bogusDatabaseName, "%", "%");
+	}
+
+	/**
+	 * Tests fix for BUG#23303 - DBMD.getSchemas() doesn't return a TABLE_CATALOG column.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug23303() throws Exception {
+		try {
+			this.rs = this.conn.getMetaData().getSchemas();
+			this.rs.findColumn("TABLE_CATALOG");
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
 	
+	/**
+	 * Tests fix for BUG#23304 - DBMD using "show" and DBMD using 
+	 * information_schema do not return results consistent with eachother.
+	 * 
+	 * (note this fix only addresses the inconsistencies, not the issue that
+	 * the driver is treating schemas differently than some users expect.
+	 * 
+	 * We will revisit this behavior when there is full support for schemas
+	 * in MySQL).
+	 * 
+	 * @throws Exception
+	 */
+	
+	public void testBug23304() throws Exception {
+		if (!versionMeetsMinimum(5, 0)) {
+			return;
+		}
+		
+		Connection connShow = null;
+		Connection connInfoSchema = null;
+		
+		ResultSet rsShow = null;
+		ResultSet rsInfoSchema = null;
+		
+		try {
+			Properties noInfoSchemaProps = new Properties();
+			noInfoSchemaProps.setProperty("useInformationSchema", "false");
+			
+			Properties infoSchemaProps = new Properties();
+			infoSchemaProps.setProperty("useInformationSchema", "true");
+			infoSchemaProps.setProperty("dumpQueriesOnException", "true");
+			
+			connShow = getConnectionWithProps(noInfoSchemaProps);
+			connInfoSchema = getConnectionWithProps(infoSchemaProps);
+			
+			DatabaseMetaData dbmdUsingShow = connShow.getMetaData();
+			DatabaseMetaData dbmdUsingInfoSchema = connInfoSchema.getMetaData();
+			
+			assertNotSame(dbmdUsingShow.getClass(), dbmdUsingInfoSchema.getClass());
+			
+			if (!isRunningOnJdk131()) {
+				rsShow = dbmdUsingShow.getSchemas();
+				rsInfoSchema = dbmdUsingInfoSchema.getSchemas();
+			
+				compareResultSets(rsShow, rsInfoSchema);	
+			}
+			
+			/*
+			rsShow = dbmdUsingShow.getTables(connShow.getCatalog(), null, "%", new String[] {"TABLE", "VIEW"});
+			rsInfoSchema = dbmdUsingInfoSchema.getTables(connInfoSchema.getCatalog(), null, "%", new String[] {"TABLE", "VIEW"});
+			
+			compareResultSets(rsShow, rsInfoSchema);
+			
+			rsShow = dbmdUsingShow.getTables(null, null, "%", new String[] {"TABLE", "VIEW"});
+			rsInfoSchema = dbmdUsingInfoSchema.getTables(null, null, "%", new String[] {"TABLE", "VIEW"});
+		
+			compareResultSets(rsShow, rsInfoSchema);
+			*/
+			
+			createTable("t_testBug23304", "(field1 int primary key not null, field2 tinyint, field3 mediumint, field4 mediumint, field5 bigint, field6 float, field7 double, field8 decimal, field9 char(32), field10 varchar(32), field11 blob, field12 mediumblob, field13 longblob, field14 text, field15 mediumtext, field16 longtext, field17 date, field18 time, field19 datetime, field20 timestamp)");
+			
+			rsShow = dbmdUsingShow.getColumns(connShow.getCatalog(), null, "t_testBug23304", "%");
+			rsInfoSchema = dbmdUsingInfoSchema.getColumns(connInfoSchema.getCatalog(), null, "t_testBug23304", "%");
+			
+			compareResultSets(rsShow, rsInfoSchema);
+		} finally {
+			if (rsShow != null) {
+				rsShow.close();
+			}
+			
+			if (rsInfoSchema != null) {
+				rsInfoSchema.close();
+			}
+		}
+	}
+	
+	private void compareResultSets(ResultSet expected, ResultSet actual) throws Exception {
+		if (expected == null && actual != null) {
+			fail("Expected null result set, actual was not null.");
+		} else if (expected != null && actual == null) {
+			fail("Expected non-null actual result set.");
+		} else if (expected == null && actual == null) {
+			return;
+		}
+		
+		expected.last();
+		
+		int expectedRows = expected.getRow();
+		
+		actual.last();
+		
+		int actualRows = actual.getRow();
+		
+		assertEquals(expectedRows, actualRows);
+		
+		ResultSetMetaData metadataExpected = expected.getMetaData();
+		ResultSetMetaData metadataActual = actual.getMetaData();
+		
+		assertEquals(metadataExpected.getColumnCount(), metadataActual.getColumnCount());
+		
+		for (int i = 0; i < metadataExpected.getColumnCount(); i++) {
+			assertEquals(metadataExpected.getColumnName(i + 1), metadataActual.getColumnName(i + 1));
+			assertEquals(metadataExpected.getColumnType(i + 1), metadataActual.getColumnType(i + 1));
+			assertEquals(metadataExpected.getColumnClassName(i + 1), metadataActual.getColumnClassName(i + 1));
+		}
+		
+		expected.beforeFirst();
+		actual.beforeFirst();
+		
+		StringBuffer messageBuf = null;
+		
+		while (expected.next() && actual.next()) {
+			
+			if (messageBuf != null) {
+				messageBuf.append("\n");
+			}
+			
+			for (int i = 0; i < metadataExpected.getColumnCount(); i++) {
+				if (expected.getObject(i + 1) == null && actual.getObject(i + 1) == null) {
+					continue;
+				}
+				
+				if ((expected.getObject(i + 1) == null && actual.getObject(i + 1) != null) ||
+						(expected.getObject(i + 1) != null && actual.getObject(i + 1) == null) ||
+						(!expected.getObject(i + 1).equals(actual.getObject(i + 1)))) {
+					if ("COLUMN_DEF".equals(metadataExpected.getColumnName(i + 1)) && 
+							(expected.getObject(i + 1) == null && actual.getString(i + 1).length() == 0) ||
+							(expected.getString(i + 1).length() == 0 && actual.getObject(i + 1) == null)) {
+						continue; // known bug with SHOW FULL COLUMNS, and we can't distinguish between null and ''
+						          // for a default
+					}
+					
+					if (messageBuf == null) {
+						messageBuf = new StringBuffer();
+					} else {
+						messageBuf.append("\n");
+					}
+					
+					messageBuf.append("On row " + expected.getRow() + " ,for column named " + metadataExpected.getColumnName(i + 1) + ", expected '" + expected.getObject(i + 1) + "', found '" + actual.getObject(i + 1) + "'");
+					
+				}
+			}
+		}
+		
+		if (messageBuf != null) {
+			fail(messageBuf.toString());
+		}
+	}
+	
+	/**
+	 * Tests fix for BUG#25624 - Whitespace surrounding storage/size specifiers in stored procedure
+	 * declaration causes NumberFormatException to be thrown when calling stored procedure.
+	 * 
+	 * @throws Exception
+	 */
+	public void testBug25624() throws Exception {
+		if (!versionMeetsMinimum(5, 0)) {
+			return;
+		}
+
+		//
+		// we changed up the parameters to get coverage of the fixes,
+		// also note that whitespace _is_ significant in the DDL...
+		//
+		
+		createProcedure(
+				"testBug25624",
+				"(in _par1 decimal( 10 , 2 ) , in _par2 varchar( 4 )) BEGIN select 1; END");
+
+		this.conn.prepareCall("{call testBug25624(?,?)}").close();
+	}
 }
